@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -13,27 +13,45 @@
 
 //! Provides the `GET /admin/proposals` endpoint for listing circuit proposals.
 
+use std::collections::HashMap;
+use std::convert::TryFrom;
+
 use actix_web::{error::BlockingError, web, Error, HttpRequest, HttpResponse};
 use futures::{future::IntoFuture, Future};
-use std::collections::HashMap;
 
-use crate::admin::service::proposal_store::{ProposalFilter, ProposalStore};
+#[cfg(feature = "authorization")]
+use crate::admin::rest_api::CIRCUIT_READ_PERMISSION;
+use crate::admin::service::proposal_store::ProposalStore;
+use crate::admin::store::CircuitPredicate;
 use crate::protocol;
-use crate::rest_api::paging::{get_response_paging_info, DEFAULT_LIMIT, DEFAULT_OFFSET};
-use crate::rest_api::{ErrorResponse, Method, ProtocolVersionRangeGuard, Resource};
+use crate::rest_api::{
+    actix_web_1::{Method, ProtocolVersionRangeGuard, Resource},
+    paging::{get_response_paging_info, DEFAULT_LIMIT, DEFAULT_OFFSET},
+    ErrorResponse,
+};
 
 use super::super::error::ProposalListError;
-use super::super::resources::proposals::{ListProposalsResponse, ProposalResponse};
+use super::super::resources;
 
 pub fn make_list_proposals_resource<PS: ProposalStore + 'static>(proposal_store: PS) -> Resource {
-    Resource::build("admin/proposals")
-        .add_request_guard(ProtocolVersionRangeGuard::new(
+    let resource =
+        Resource::build("admin/proposals").add_request_guard(ProtocolVersionRangeGuard::new(
             protocol::ADMIN_LIST_PROPOSALS_PROTOCOL_MIN,
             protocol::ADMIN_PROTOCOL_VERSION,
-        ))
-        .add_method(Method::Get, move |r, _| {
+        ));
+
+    #[cfg(feature = "authorization")]
+    {
+        resource.add_method(Method::Get, CIRCUIT_READ_PERMISSION, move |r, _| {
             list_proposals(r, web::Data::new(proposal_store.clone()))
         })
+    }
+    #[cfg(not(feature = "authorization"))]
+    {
+        resource.add_method(Method::Get, move |r, _| {
+            list_proposals(r, web::Data::new(proposal_store.clone()))
+        })
+    }
 }
 
 fn list_proposals<PS: ProposalStore + 'static>(
@@ -100,6 +118,22 @@ fn list_proposals<PS: ProposalStore + 'static>(
         link.push_str(&format!("?{}&", new_queries.join("&")));
     }
 
+    let protocol_version = match req.headers().get("SplinterProtocolVersion") {
+        Some(header_value) => match header_value.to_str() {
+            Ok(protocol_version) => protocol_version.to_string(),
+            Err(_) => {
+                return Box::new(
+                    HttpResponse::BadRequest()
+                        .json(ErrorResponse::bad_request(
+                            "Unable to get SplinterProtocolVersion",
+                        ))
+                        .into_future(),
+                )
+            }
+        },
+        None => format!("{}", protocol::ADMIN_PROTOCOL_VERSION),
+    };
+
     Box::new(query_list_proposals(
         proposal_store,
         link,
@@ -107,6 +141,7 @@ fn list_proposals<PS: ProposalStore + 'static>(
         member_filter,
         Some(offset),
         Some(limit),
+        protocol_version,
     ))
 }
 
@@ -117,14 +152,15 @@ fn query_list_proposals<PS: ProposalStore + 'static>(
     member_filter: Option<String>,
     offset: Option<usize>,
     limit: Option<usize>,
+    protocol_version: String,
 ) -> impl Future<Item = HttpResponse, Error = Error> {
     web::block(move || {
         let mut filters = vec![];
         if let Some(management_type) = management_type_filter {
-            filters.push(ProposalFilter::WithManagementType(management_type));
+            filters.push(CircuitPredicate::ManagementTypeEq(management_type));
         }
         if let Some(member) = member_filter {
-            filters.push(ProposalFilter::WithMember(member));
+            filters.push(CircuitPredicate::MembersInclude(vec![member]));
         }
 
         let proposals = proposal_store
@@ -139,14 +175,42 @@ fn query_list_proposals<PS: ProposalStore + 'static>(
             .take(limit_value)
             .collect::<Vec<_>>();
 
-        Ok((proposals, link, limit, offset, total))
+        Ok((proposals, link, limit, offset, total, protocol_version))
     })
     .then(|res| match res {
-        Ok((proposals, link, limit, offset, total_count)) => {
-            Ok(HttpResponse::Ok().json(ListProposalsResponse {
-                data: proposals.iter().map(ProposalResponse::from).collect(),
-                paging: get_response_paging_info(limit, offset, &link, total_count),
-            }))
+        Ok((proposals, link, limit, offset, total_count, protocol_version)) => {
+            match protocol_version.as_str() {
+                "1" => Ok(HttpResponse::Ok().json(
+                    resources::v1::proposals::ListProposalsResponse {
+                        data: proposals
+                            .iter()
+                            .map(resources::v1::proposals::ProposalResponse::from)
+                            .collect(),
+                        paging: get_response_paging_info(limit, offset, &link, total_count),
+                    },
+                )),
+                // Handles 2 (and catch all)
+                _ => {
+                    let proposal_responses = match proposals
+                        .iter()
+                        .map(resources::v2::proposals::ProposalResponse::try_from)
+                        .collect::<Result<
+                            Vec<resources::v2::proposals::ProposalResponse>, &'static str>>()
+                    {
+                        Ok(proposal) => proposal,
+                        Err(err) => {
+                            error!("{}", err);
+                            return Ok(HttpResponse::InternalServerError().into());
+                        }
+                    };
+                    Ok(
+                        HttpResponse::Ok().json(resources::v2::proposals::ListProposalsResponse {
+                            data: proposal_responses,
+                            paging: get_response_paging_info(limit, offset, &link, total_count),
+                        }),
+                    )
+                }
+            }
         }
         Err(err) => match err {
             BlockingError::Error(err) => match err {
@@ -169,13 +233,18 @@ mod tests {
 
     use crate::admin::{
         messages::{
-            AuthorizationType, CircuitProposal, CreateCircuit, DurabilityType, PersistenceType,
-            ProposalType, RouteType, SplinterNode,
+            AuthorizationType, CircuitProposal, CircuitStatus, CreateCircuit, DurabilityType,
+            PersistenceType, ProposalType, RouteType, SplinterNode,
+        },
+        service::proposal_store::{ProposalIter, ProposalStoreError},
+        store::{
+            self, CircuitPredicate, CircuitProposal as StoreProposal, CircuitProposalBuilder,
+            ProposedCircuitBuilder, ProposedNodeBuilder,
         },
-        service::proposal_store::{ProposalFilter, ProposalIter, ProposalStoreError},
     };
     use crate::rest_api::{
-        paging::Paging, RestApiBuilder, RestApiServerError, RestApiShutdownHandle,
+        actix_web_1::{RestApiBuilder, RestApiShutdownHandle},
+        paging::Paging,
     };
 
     #[test]
@@ -197,9 +266,73 @@ mod tests {
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
             &to_value(vec![
-                ProposalResponse::from(&get_proposal_1()),
-                ProposalResponse::from(&get_proposal_2()),
-                ProposalResponse::from(&get_proposal_3()),
+                resources::v2::proposals::ProposalResponse::try_from(&CircuitProposal::from(
+                    get_proposal_1()
+                ))
+                .expect("Unable to get ProposalResponse"),
+                resources::v2::proposals::ProposalResponse::try_from(&CircuitProposal::from(
+                    get_proposal_2()
+                ))
+                .expect("Unable to get ProposalResponse"),
+                resources::v2::proposals::ProposalResponse::try_from(&CircuitProposal::from(
+                    get_proposal_3()
+                ))
+                .expect("Unable to get ProposalResponse"),
+            ])
+            .expect("failed to convert expected data"),
+        );
+
+        assert_eq!(
+            proposals
+                .get("paging")
+                .expect("no paging field in response"),
+            &to_value(create_test_paging_response(
+                0,
+                100,
+                0,
+                0,
+                0,
+                3,
+                "/admin/proposals?"
+            ))
+            .expect("failed to convert expected paging")
+        );
+
+        shutdown_handle
+            .shutdown()
+            .expect("unable to shutdown rest api");
+        join_handle.join().expect("Unable to join rest api thread");
+    }
+
+    #[test]
+    /// Tests a GET /admin/proposals request with protocol 1 and no filters returns the expected
+    /// proposals. This test is for backwards compatibility.
+    fn test_list_proposals_ok_v1() {
+        let (shutdown_handle, join_handle, bind_url) =
+            run_rest_api_on_open_port(vec![make_list_proposals_resource(MockProposalStore)]);
+
+        let url = Url::parse(&format!("http://{}/admin/proposals", bind_url))
+            .expect("Failed to parse URL");
+        let req = Client::new()
+            .get(url)
+            .header("SplinterProtocolVersion", "1");
+        let resp = req.send().expect("Failed to perform request");
+
+        assert_eq!(resp.status(), StatusCode::OK);
+        let proposals: JsonValue = resp.json().expect("Failed to deserialize body");
+
+        assert_eq!(
+            proposals.get("data").expect("no data field in response"),
+            &to_value(vec![
+                resources::v1::proposals::ProposalResponse::from(&CircuitProposal::from(
+                    get_proposal_1()
+                )),
+                resources::v1::proposals::ProposalResponse::from(&CircuitProposal::from(
+                    get_proposal_2()
+                )),
+                resources::v1::proposals::ProposalResponse::from(&CircuitProposal::from(
+                    get_proposal_3()
+                )),
             ])
             .expect("failed to convert expected data"),
         );
@@ -248,8 +381,11 @@ mod tests {
 
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
-            &to_value(vec![ProposalResponse::from(&get_proposal_1())])
-                .expect("failed to convert expected data"),
+            &to_value(vec![resources::v2::proposals::ProposalResponse::try_from(
+                &get_proposal_1()
+            )
+            .expect("Unable to get ProposalResponse")])
+            .expect("failed to convert expected data"),
         );
 
         assert_eq!(
@@ -297,8 +433,10 @@ mod tests {
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
             &to_value(vec![
-                ProposalResponse::from(&get_proposal_1()),
-                ProposalResponse::from(&get_proposal_3())
+                resources::v2::proposals::ProposalResponse::try_from(&get_proposal_1())
+                    .expect("Unable to get ProposalResponse"),
+                resources::v2::proposals::ProposalResponse::try_from(&get_proposal_3())
+                    .expect("Unable to get ProposalResponse")
             ])
             .expect("failed to convert expected data"),
         );
@@ -347,8 +485,11 @@ mod tests {
 
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
-            &to_value(vec![ProposalResponse::from(&get_proposal_3())])
-                .expect("failed to convert expected data"),
+            &to_value(vec![resources::v2::proposals::ProposalResponse::try_from(
+                &get_proposal_3()
+            )
+            .expect("Unable to get ProposalResponse")])
+            .expect("failed to convert expected data"),
         );
 
         assert_eq!(
@@ -391,8 +532,11 @@ mod tests {
 
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
-            &to_value(vec![ProposalResponse::from(&get_proposal_1())])
-                .expect("failed to convert expected data"),
+            &to_value(vec![resources::v2::proposals::ProposalResponse::try_from(
+                &get_proposal_1()
+            )
+            .expect("Unable to get ProposalResponse")])
+            .expect("failed to convert expected data"),
         );
 
         assert_eq!(
@@ -436,8 +580,10 @@ mod tests {
         assert_eq!(
             proposals.get("data").expect("no data field in response"),
             &to_value(vec![
-                ProposalResponse::from(&get_proposal_2()),
-                ProposalResponse::from(&get_proposal_3())
+                resources::v2::proposals::ProposalResponse::try_from(&get_proposal_2())
+                    .expect("Unable to get ProposalResponse"),
+                resources::v2::proposals::ProposalResponse::try_from(&get_proposal_3())
+                    .expect("Unable to get ProposalResponse")
             ])
             .expect("failed to convert expected data"),
         );
@@ -498,21 +644,17 @@ mod tests {
     impl ProposalStore for MockProposalStore {
         fn proposals(
             &self,
-            filters: Vec<ProposalFilter>,
+            filters: Vec<CircuitPredicate>,
         ) -> Result<ProposalIter, ProposalStoreError> {
-            let proposals = vec![get_proposal_1(), get_proposal_2(), get_proposal_3()];
+            let mut proposals = get_proposal_list();
 
-            let total = proposals
-                .iter()
-                .filter(|proposal| filters.iter().all(|filter| filter.matches(&proposal)))
-                .count();
+            proposals.retain(|proposal| {
+                filters
+                    .iter()
+                    .all(|predicate| predicate.apply_to_proposals(proposal))
+            });
 
-            let iter =
-                Box::new(proposals.into_iter().filter(move |proposal| {
-                    filters.iter().all(|filter| filter.matches(&proposal))
-                }));
-
-            Ok(ProposalIter::new(iter, total))
+            Ok(ProposalIter::new(Box::new(proposals.into_iter())))
         }
 
         fn proposal(
@@ -523,13 +665,95 @@ mod tests {
         }
     }
 
+    fn get_proposal_list() -> Vec<StoreProposal> {
+        vec![
+            CircuitProposalBuilder::new()
+                .with_proposal_type(&store::ProposalType::Create)
+                .with_circuit_id("abcDE-00000")
+                .with_circuit_hash("012345")
+                .with_circuit(
+                    &ProposedCircuitBuilder::new()
+                        .with_circuit_id("abcDE-00000")
+                        .with_roster(&[])
+                        .with_members(&[ProposedNodeBuilder::new()
+                            .with_node_id("node_id")
+                            .with_endpoints(&["".into()])
+                            .build()
+                            .expect("Unable to build circuit node")])
+                        .with_authorization_type(&store::AuthorizationType::Trust)
+                        .with_persistence(&store::PersistenceType::Any)
+                        .with_durability(&store::DurabilityType::NoDurability)
+                        .with_routes(&store::RouteType::Any)
+                        .with_circuit_management_type("mgmt_type_1")
+                        .with_comments("mock circuit 1")
+                        .with_display_name("circuit_1")
+                        .build()
+                        .expect("Unable to create proposed circuit"),
+                )
+                .with_requester(&[])
+                .with_requester_node_id("node_id")
+                .build()
+                .expect("unable to build proposal"),
+            CircuitProposalBuilder::new()
+                .with_proposal_type(&store::ProposalType::Create)
+                .with_circuit_id("abcDE-00001")
+                .with_circuit_hash("abcdef")
+                .with_circuit(
+                    &ProposedCircuitBuilder::new()
+                        .with_circuit_id("abcDE-00001")
+                        .with_roster(&[])
+                        .with_members(&[])
+                        .with_authorization_type(&store::AuthorizationType::Trust)
+                        .with_persistence(&store::PersistenceType::Any)
+                        .with_durability(&store::DurabilityType::NoDurability)
+                        .with_routes(&store::RouteType::Any)
+                        .with_circuit_management_type("mgmt_type_2")
+                        .with_comments("mock circuit 2")
+                        .with_display_name("circuit_2")
+                        .with_circuit_version(2)
+                        .build()
+                        .expect("Unable to create proposed circuit"),
+                )
+                .with_requester(&[])
+                .with_requester_node_id("node_id")
+                .build()
+                .expect("unable to build proposal"),
+            CircuitProposalBuilder::new()
+                .with_proposal_type(&store::ProposalType::Create)
+                .with_circuit_id("abcDE-00002")
+                .with_circuit_hash("678910")
+                .with_circuit(
+                    &ProposedCircuitBuilder::new()
+                        .with_circuit_id("abcDE-00002")
+                        .with_roster(&[])
+                        .with_members(&[ProposedNodeBuilder::new()
+                            .with_node_id("node_id")
+                            .with_endpoints(&["".into()])
+                            .build()
+                            .expect("Unable to build circuit node")])
+                        .with_authorization_type(&store::AuthorizationType::Trust)
+                        .with_persistence(&store::PersistenceType::Any)
+                        .with_durability(&store::DurabilityType::NoDurability)
+                        .with_routes(&store::RouteType::Any)
+                        .with_circuit_management_type("mgmt_type_2")
+                        .with_comments("mock circuit 3")
+                        .build()
+                        .expect("Unable to create proposed circuit"),
+                )
+                .with_requester(&[])
+                .with_requester_node_id("node_id")
+                .build()
+                .expect("unable to build proposal"),
+        ]
+    }
+
     fn get_proposal_1() -> CircuitProposal {
         CircuitProposal {
             proposal_type: ProposalType::Create,
-            circuit_id: "circuit1".into(),
+            circuit_id: "abcDE-00000".into(),
             circuit_hash: "012345".into(),
             circuit: CreateCircuit {
-                circuit_id: "circuit1".into(),
+                circuit_id: "abcDE-00000".into(),
                 roster: vec![],
                 members: vec![SplinterNode {
                     node_id: "node_id".into(),
@@ -541,7 +765,10 @@ mod tests {
                 routes: RouteType::Any,
                 circuit_management_type: "mgmt_type_1".into(),
                 application_metadata: vec![],
-                comments: "mock circuit 1".into(),
+                comments: Some("mock circuit 1".into()),
+                display_name: Some("circuit_1".into()),
+                circuit_version: 1,
+                circuit_status: CircuitStatus::Active,
             },
             votes: vec![],
             requester: vec![],
@@ -552,10 +779,10 @@ mod tests {
     fn get_proposal_2() -> CircuitProposal {
         CircuitProposal {
             proposal_type: ProposalType::Create,
-            circuit_id: "circuit2".into(),
+            circuit_id: "abcDE-00001".into(),
             circuit_hash: "abcdef".into(),
             circuit: CreateCircuit {
-                circuit_id: "circuit2".into(),
+                circuit_id: "abcDE-00001".into(),
                 roster: vec![],
                 members: vec![],
                 authorization_type: AuthorizationType::Trust,
@@ -564,7 +791,10 @@ mod tests {
                 routes: RouteType::Any,
                 circuit_management_type: "mgmt_type_2".into(),
                 application_metadata: vec![],
-                comments: "mock circuit 2".into(),
+                comments: Some("mock circuit 2".into()),
+                display_name: Some("circuit_2".into()),
+                circuit_version: 2,
+                circuit_status: CircuitStatus::Active,
             },
             votes: vec![],
             requester: vec![],
@@ -575,10 +805,10 @@ mod tests {
     fn get_proposal_3() -> CircuitProposal {
         CircuitProposal {
             proposal_type: ProposalType::Create,
-            circuit_id: "circuit3".into(),
+            circuit_id: "abcDE-00002".into(),
             circuit_hash: "678910".into(),
             circuit: CreateCircuit {
-                circuit_id: "circuit3".into(),
+                circuit_id: "abcDE-00002".into(),
                 roster: vec![],
                 members: vec![SplinterNode {
                     node_id: "node_id".into(),
@@ -590,7 +820,10 @@ mod tests {
                 routes: RouteType::Any,
                 circuit_management_type: "mgmt_type_2".into(),
                 application_metadata: vec![],
-                comments: "mock circuit 3".into(),
+                comments: Some("mock circuit 3".into()),
+                display_name: None,
+                circuit_version: 1,
+                circuit_status: CircuitStatus::Active,
             },
             votes: vec![],
             requester: vec![],
@@ -601,23 +834,23 @@ mod tests {
     fn run_rest_api_on_open_port(
         resources: Vec<Resource>,
     ) -> (RestApiShutdownHandle, std::thread::JoinHandle<()>, String) {
-        (10000..20000)
-            .find_map(|port| {
-                let bind_url = format!("127.0.0.1:{}", port);
-                let result = RestApiBuilder::new()
-                    .with_bind(&bind_url)
-                    .add_resources(resources.clone())
-                    .build()
-                    .expect("Failed to build REST API")
-                    .run();
-                match result {
-                    Ok((shutdown_handle, join_handle)) => {
-                        Some((shutdown_handle, join_handle, bind_url))
-                    }
-                    Err(RestApiServerError::BindError(_)) => None,
-                    Err(err) => panic!("Failed to run REST API: {}", err),
-                }
-            })
-            .expect("No port available")
+        #[cfg(not(feature = "https-bind"))]
+        let bind = "127.0.0.1:0";
+        #[cfg(feature = "https-bind")]
+        let bind = crate::rest_api::BindConfig::Http("127.0.0.1:0".into());
+
+        let result = RestApiBuilder::new()
+            .with_bind(bind)
+            .add_resources(resources.clone())
+            .build_insecure()
+            .expect("Failed to build REST API")
+            .run_insecure();
+        match result {
+            Ok((shutdown_handle, join_handle)) => {
+                let port = shutdown_handle.port_numbers()[0];
+                (shutdown_handle, join_handle, format!("127.0.0.1:{}", port))
+            }
+            Err(err) => panic!("Failed to run REST API: {}", err),
+        }
     }
 }
