@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,31 +12,33 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+mod builder;
 mod consensus;
 pub(crate) mod error;
-mod mailbox;
 pub(crate) mod messages;
-pub(super) mod open_proposals;
 pub(super) mod proposal_store;
 mod shared;
+mod subscriber;
 
 use std::any::Any;
 #[cfg(feature = "service-arg-validation")]
 use std::collections::HashMap;
 use std::sync::{mpsc::channel, Arc, Mutex};
-use std::thread;
-use std::time::{Duration, SystemTime};
+use std::thread::{self, JoinHandle};
+use std::time::Duration;
 
+use cylinder::Verifier as SignatureVerifier;
 use openssl::hash::{hash, MessageDigest};
 use protobuf::{self, Message};
 
-use crate::circuit::SplinterState;
+use crate::admin::store::{self, AdminServiceStore};
+use crate::circuit::routing::{self, RoutingTableWriter};
 use crate::consensus::Proposal;
 use crate::hex::to_hex;
 use crate::keys::KeyPermissionManager;
 use crate::orchestrator::{ServiceDefinition, ServiceOrchestrator};
 use crate::peer::{PeerManagerConnector, PeerManagerNotification};
-use crate::protocol::{ADMIN_PROTOCOL_VERSION, ADMIN_SERVICE_PROTOCOL_MIN};
+use crate::protocol::{ADMIN_SERVICE_PROTOCOL_MIN, ADMIN_SERVICE_PROTOCOL_VERSION};
 use crate::protos::admin::{
     AdminMessage, AdminMessage_Type, CircuitManagementPayload, ServiceProtocolVersionResponse,
 };
@@ -48,27 +50,18 @@ use crate::service::{
     error::{ServiceDestroyError, ServiceError, ServiceStartError, ServiceStopError},
     Service, ServiceMessageContext, ServiceNetworkRegistry,
 };
-use crate::signing::SignatureVerifier;
 
 use self::consensus::AdminConsensusManager;
 use self::error::{AdminError, Sha256Error};
 use self::proposal_store::{AdminServiceProposals, ProposalStore};
 use self::shared::AdminServiceShared;
 
+pub use self::builder::AdminServiceBuilder;
 pub use self::error::AdminKeyVerifierError;
 pub use self::error::AdminServiceError;
 pub use self::error::AdminSubscriberError;
 pub use self::shared::AdminServiceStatus;
-
-const DEFAULT_COORDINATOR_TIMEOUT: u64 = 30; // 30 seconds
-
-pub trait AdminServiceEventSubscriber: Send {
-    fn handle_event(
-        &self,
-        admin_service_event: &messages::AdminServiceEvent,
-        timestamp: &SystemTime,
-    ) -> Result<(), AdminSubscriberError>;
-}
+pub use self::subscriber::AdminServiceEventSubscriber;
 
 pub trait AdminCommands: Send + Sync {
     fn submit_circuit_change(
@@ -84,7 +77,7 @@ pub trait AdminCommands: Send + Sync {
 
     fn get_events_since(
         &self,
-        since_timestamp: &SystemTime,
+        since_event_id: &i64,
         event_type: &str,
     ) -> Result<Events, AdminServiceError>;
 
@@ -110,7 +103,7 @@ impl AdminKeyVerifier for dyn RegistryReader {
     /// The key is permitted if and only if the node with the given `node_id` exists in the
     /// registry and the node has the given key. Otherwise, the key is not permitted.
     fn is_permitted(&self, node_id: &str, key: &[u8]) -> Result<bool, AdminKeyVerifierError> {
-        let node_opt = self.fetch_node(node_id).map_err(|err| {
+        let node_opt = self.get_node(node_id).map_err(|err| {
             AdminKeyVerifierError::new_with_source(
                 &format!("Failed to lookup node '{}' in registry", node_id),
                 Box::new(err),
@@ -130,13 +123,12 @@ impl AdminKeyVerifier for Box<dyn RegistryReader> {
     }
 }
 
-/// An iterator over AdminServiceEvents and the time that each occurred.
 pub struct Events {
-    inner: Box<dyn Iterator<Item = (SystemTime, messages::AdminServiceEvent)> + Send>,
+    inner: Box<dyn ExactSizeIterator<Item = store::AdminServiceEvent> + Send>,
 }
 
 impl Iterator for Events {
-    type Item = (SystemTime, messages::AdminServiceEvent);
+    type Item = store::AdminServiceEvent;
 
     fn next(&mut self) -> Option<Self::Item> {
         self.inner.next()
@@ -152,10 +144,13 @@ pub struct AdminService {
     coordinator_timeout: Duration,
     consensus: Option<AdminConsensusManager>,
     peer_connector: PeerManagerConnector,
+
+    peer_notification_run_state: Option<(usize, JoinHandle<()>)>,
 }
 
 impl AdminService {
     #![allow(clippy::too_many_arguments)]
+    #[deprecated(since = "0.5.1", note = "please use `AdminServiceBuilder` instead")]
     pub fn new(
         node_id: &str,
         orchestrator: ServiceOrchestrator,
@@ -164,73 +159,39 @@ impl AdminService {
             Box<dyn ServiceArgValidator + Send>,
         >,
         peer_connector: PeerManagerConnector,
-        splinter_state: SplinterState,
-        signature_verifier: Box<dyn SignatureVerifier + Send>,
+        admin_store: Box<dyn AdminServiceStore>,
+        signature_verifier: Box<dyn SignatureVerifier>,
         key_verifier: Box<dyn AdminKeyVerifier>,
         key_permission_manager: Box<dyn KeyPermissionManager>,
-        storage_type: &str,
-        state_dir: &str,
         // The coordinator timeout for the two-phase commit consensus engine; if `None`, the
         // default value will be used (30 seconds).
         coordinator_timeout: Option<Duration>,
-    ) -> Result<(Self, thread::JoinHandle<()>), ServiceError> {
-        let coordinator_timeout =
-            coordinator_timeout.unwrap_or_else(|| Duration::from_secs(DEFAULT_COORDINATOR_TIMEOUT));
-        let orchestrator = Arc::new(Mutex::new(orchestrator));
-        let (sender, receiver) = channel();
-        peer_connector
-            .subscribe_sender(sender)
-            .map_err(|err| ServiceError::UnableToCreate(Box::new(err)))?;
+        routing_table_writer: Box<dyn RoutingTableWriter>,
+        event_store: Box<dyn AdminServiceStore>,
+    ) -> Result<Self, ServiceError> {
+        let mut builder = builder::AdminServiceBuilder::new()
+            .with_node_id(node_id.to_string())
+            .with_service_orchestrator(orchestrator)
+            .with_peer_manager_connector(peer_connector)
+            .with_admin_service_store(admin_store)
+            .with_signature_verifier(signature_verifier)
+            .with_admin_key_verifier(key_verifier)
+            .with_key_permission_manager(key_permission_manager)
+            .with_routing_table_writer(routing_table_writer)
+            .with_admin_event_store(event_store);
 
-        let new_service = Self {
-            service_id: admin_service_id(node_id),
-            node_id: node_id.to_string(),
-            admin_service_shared: Arc::new(Mutex::new(AdminServiceShared::new(
-                node_id.to_string(),
-                orchestrator.clone(),
-                #[cfg(feature = "service-arg-validation")]
-                service_arg_validators,
-                peer_connector.clone(),
-                splinter_state,
-                signature_verifier,
-                key_verifier,
-                key_permission_manager,
-                storage_type,
-                state_dir,
-            )?)),
-            orchestrator,
-            coordinator_timeout,
-            consensus: None,
-            peer_connector,
-        };
+        if let Some(coordinator_timeout) = coordinator_timeout {
+            builder = builder.with_coordinator_timeout(coordinator_timeout);
+        }
 
-        let peer_admin_shared = new_service.admin_service_shared.clone();
+        #[cfg(feature = "service-arg-validation")]
+        {
+            builder = builder.with_service_arg_validators(service_arg_validators);
+        }
 
-        debug!("Starting admin service's peer manager notification receiver");
-        let notification_join_handle = thread::Builder::new()
-            .name("PeerManagerNotification Receiver".into())
-            .spawn(move || loop {
-                let notification = match receiver.recv() {
-                    Ok(notification) => notification,
-                    Err(_) => {
-                        warn!(
-                            "Admin service received an error while listening to peer manager \
-                            notifications, indicating remote thread has shutdown"
-                        );
-                        break;
-                    }
-                };
-
-                if let Ok(mut admin_shared) = peer_admin_shared.lock() {
-                    handle_peer_manager_notification(notification, &mut *admin_shared);
-                } else {
-                    error!("the admin shared lock was poisoned");
-                    break;
-                }
-            })
-            .map_err(|err| ServiceError::UnableToCreate(Box::new(err)))?;
-
-        Ok((new_service, notification_join_handle))
+        builder
+            .build()
+            .map_err(|e| ServiceError::UnableToCreate(Box::new(e)))
     }
 
     pub fn commands(&self) -> impl AdminCommands + Clone {
@@ -247,17 +208,44 @@ impl AdminService {
     /// circuits should be initialized using the service orchestrator. This may not include all
     /// services if they are not supported locally. It is expected that some services will be
     /// started externally.
+    /// Furthermore, this adds services from circuits that have previously stopped their services
+    /// to the orchestrator using the `add_stopped_service` which allows for the service data to
+    /// be removed if the inactive circuit is purged. The separate handling of the inactive circuits
+    /// is necessary in order to avoid adding networking functionality to these circuits, as they
+    /// have already had this functionality removed through disbanding or abandoning.
     ///
     /// Also adds peer references for members of the circuits and proposals.
     fn re_initialize_circuits(&self) -> Result<(), ServiceStartError> {
-        let circuits = self
+        #[cfg(feature = "admin-service-count")]
+        self.admin_service_shared
+            .lock()
+            .map_err(|_| {
+                ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
+            })?
+            .update_metrics()
+            .map_err(|err| {
+                ServiceStartError::Internal(format!("Unable to update metrics: {}", err))
+            })?;
+
+        let mut active_circuits = vec![];
+        let mut inactive_circuits = vec![];
+        for circuit in self
             .admin_service_shared
             .lock()
             .map_err(|_| {
                 ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
             })?
             .get_circuits()
-            .map_err(|err| ServiceStartError::Internal(Box::new(err)))?;
+            .map_err(|err| {
+                ServiceStartError::Internal(format!("Unable to get circuits: {}", err))
+            })?
+        {
+            if circuit.circuit_status() == &store::CircuitStatus::Active {
+                active_circuits.push(circuit);
+            } else {
+                inactive_circuits.push(circuit);
+            }
+        }
 
         let nodes = self
             .admin_service_shared
@@ -266,16 +254,25 @@ impl AdminService {
                 ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
             })?
             .get_nodes()
-            .map_err(|err| ServiceStartError::Internal(Box::new(err)))?;
+            .map_err(|err| ServiceStartError::Internal(format!("Unable to get nodes: {}", err)))?;
 
         let orchestrator = self.orchestrator.lock().map_err(|_| {
             ServiceStartError::PoisonedLock("the admin orchestrator lock was poisoned".into())
         })?;
         let mut peer_refs = vec![];
         // start all services of the supported types
-        for (circuit_name, circuit) in circuits.iter() {
+        let mut writer = self
+            .admin_service_shared
+            .lock()
+            .map_err(|_| {
+                ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
+            })?
+            .routing_table_writer();
+
+        for circuit in active_circuits {
+            let mut routing_members = vec![];
             // restart all peer in the circuit
-            for member in circuit.members() {
+            for member in circuit.members().iter() {
                 if member != &self.node_id {
                     if let Some(node) = nodes.get(member) {
                         let peer_ref = self
@@ -287,6 +284,11 @@ impl AdminService {
                         } else {
                             info!("Unable to peer with {} at this time", member);
                         }
+
+                        routing_members.push(routing::CircuitNode::new(
+                            member.to_string(),
+                            node.endpoints().to_vec(),
+                        ))
                     } else {
                         error!("Missing node information for {}", member);
                     }
@@ -294,21 +296,47 @@ impl AdminService {
             }
 
             // Get all services this node is allowed to run and the orchestrator has a factory for
+
+            let routing_services = circuit
+                .roster()
+                .iter()
+                .map(|service| {
+                    routing::Service::new(
+                        service.service_id().to_string(),
+                        service.service_type().to_string(),
+                        service.node_id().to_string(),
+                        service.arguments().to_vec(),
+                    )
+                })
+                .collect();
+
             let services = circuit
                 .roster()
                 .iter()
                 .filter(|service| {
-                    service.allowed_nodes().contains(&self.node_id)
+                    service.node_id() == self.node_id
                         && orchestrator
                             .supported_service_types()
                             .contains(&service.service_type().to_string())
                 })
                 .collect::<Vec<_>>();
 
+            writer
+                .add_circuit(
+                    circuit.circuit_id().to_string(),
+                    routing::Circuit::new(
+                        circuit.circuit_id().to_string(),
+                        routing_services,
+                        circuit.members().to_vec(),
+                    ),
+                    routing_members,
+                )
+                .map_err(|err| ServiceStartError::Internal(err.reduce_to_string()))?;
+
             // Start all services
             for service in services {
                 let service_definition = ServiceDefinition {
-                    circuit: circuit_name.into(),
+                    circuit: circuit.circuit_id().into(),
                     service_id: service.service_id().into(),
                     service_type: service.service_type().into(),
                 };
@@ -325,7 +353,46 @@ impl AdminService {
                     error!(
                         "Unable to start service {} on circuit {}: {}",
                         service.service_id(),
-                        circuit_name,
+                        circuit.circuit_id(),
+                        err
+                    );
+                }
+            }
+        }
+
+        for circuit in inactive_circuits {
+            // Get all services this node is allowed to run and the orchestrator has a factory for
+            let services = circuit
+                .roster()
+                .iter()
+                .filter(|service| {
+                    service.node_id() == self.node_id
+                        && orchestrator
+                            .supported_service_types()
+                            .contains(&service.service_type().to_string())
+                })
+                .collect::<Vec<_>>();
+            // Add all services from the inactive circuits to the orchestrator
+            for service in services {
+                let service_definition = ServiceDefinition {
+                    circuit: circuit.circuit_id().into(),
+                    service_id: service.service_id().into(),
+                    service_type: service.service_type().into(),
+                };
+
+                let service_arguments = service
+                    .arguments()
+                    .iter()
+                    .map(|(key, value)| (key.clone(), value.clone()))
+                    .collect();
+
+                if let Err(err) =
+                    orchestrator.add_stopped_service(service_definition.clone(), service_arguments)
+                {
+                    error!(
+                        "Unable to add service {} from circuit {}: {}",
+                        service.service_id(),
+                        circuit.circuit_id(),
                         err
                     );
                 }
@@ -338,20 +405,23 @@ impl AdminService {
             .map_err(|_| {
                 ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
             })?
-            .get_proposals();
+            .get_proposals(&[])
+            .map_err(|err| {
+                ServiceStartError::Internal(format!("Unable to get circuit proposals: {}", err))
+            })?;
 
-        for (_, proposal) in proposals.iter() {
-            // restart all peer in the circuit
-            for member in proposal.circuit.members.iter() {
-                if member.node_id != self.node_id {
+        for proposal in proposals {
+            // connect to all peers in the circuit proposal
+            for member in proposal.circuit().members().iter() {
+                if member.node_id() != self.node_id {
                     let peer_ref = self
                         .peer_connector
-                        .add_peer_ref(member.node_id.to_string(), member.endpoints.to_vec());
+                        .add_peer_ref(member.node_id().to_string(), member.endpoints().to_vec());
 
                     if let Ok(peer_ref) = peer_ref {
                         peer_refs.push(peer_ref);
                     } else {
-                        info!("Unable to peer with {} at this time", member.node_id);
+                        info!("Unable to peer with {} at this time", member.node_id());
                     }
                 }
             }
@@ -394,13 +464,50 @@ impl Service for AdminService {
             admin_service_shared.set_network_sender(Some(network_sender));
         }
 
+        let (sender, receiver) = channel();
+        let peer_subscriber_id = self
+            .peer_connector
+            .subscribe_sender(sender)
+            .map_err(|err| ServiceStartError::Internal(err.to_string()))?;
+
+        let peer_admin_shared = self.admin_service_shared.clone();
+
+        debug!("Starting admin service's peer manager notification receiver");
+        let notification_join_handle = thread::Builder::new()
+            .name("PeerManagerNotification Receiver".into())
+            .spawn(move || loop {
+                let notification = match receiver.recv() {
+                    Ok(notification) => notification,
+                    Err(_) => {
+                        warn!(
+                            "Admin service received an error while listening to peer manager \
+                            notifications, indicating remote thread has shutdown"
+                        );
+                        break;
+                    }
+                };
+
+                if let Ok(mut admin_shared) = peer_admin_shared.lock() {
+                    handle_peer_manager_notification(notification, &mut *admin_shared);
+                } else {
+                    error!("the admin shared lock was poisoned");
+                    break;
+                }
+            })
+            .map_err(|err| ServiceStartError::Internal(err.to_string()))?;
+
+        self.peer_notification_run_state = Some((peer_subscriber_id, notification_join_handle));
+
         // Setup consensus
         let consensus = AdminConsensusManager::new(
             self.service_id().into(),
             self.admin_service_shared.clone(),
             self.coordinator_timeout,
         )
-        .map_err(|err| ServiceStartError::Internal(Box::new(err)))?;
+        .map_err(|err| {
+            ServiceStartError::Internal(format!("Unable to start consensus: {}", err))
+        })?;
+
         let proposal_sender = consensus.proposal_update_sender();
 
         self.consensus = Some(consensus);
@@ -414,14 +521,6 @@ impl Service for AdminService {
 
         self.re_initialize_circuits()?;
 
-        self.admin_service_shared
-            .lock()
-            .map_err(|_| {
-                ServiceStartError::PoisonedLock("the admin shared lock was poisoned".into())
-            })?
-            .add_services_to_directory()
-            .map_err(|err| ServiceStartError::Internal(Box::new(err)))?;
-
         self.admin_service_shared
             .lock()
             .map_err(|_| {
@@ -481,6 +580,21 @@ impl Service for AdminService {
             })?
             .change_status();
 
+        if let Some((peer_subscriber_id, peer_notfication_join_handle)) =
+            self.peer_notification_run_state.take()
+        {
+            if let Err(err) = self.peer_connector.unsubscribe(peer_subscriber_id) {
+                warn!(
+                    "Unable to unsubscribe from peer manager notifications: {:?}",
+                    err
+                );
+            }
+
+            if let Err(err) = peer_notfication_join_handle.join() {
+                error!("Failed to join peer notification thread: {:?}", err);
+            }
+        }
+
         info!("Admin service stopped and disconnected");
 
         Ok(())
@@ -494,6 +608,10 @@ impl Service for AdminService {
         }
     }
 
+    fn purge(&mut self) -> Result<(), crate::error::InternalError> {
+        Ok(())
+    }
+
     fn handle_message(
         &self,
         message_bytes: &[u8],
@@ -586,7 +704,9 @@ impl Service for AdminService {
                     ServiceError::PoisonedLock("the admin shared lock was poisoned".into())
                 })?;
 
-                if protocol > ADMIN_PROTOCOL_VERSION || protocol < ADMIN_SERVICE_PROTOCOL_MIN {
+                if !(ADMIN_SERVICE_PROTOCOL_MIN..=ADMIN_SERVICE_PROTOCOL_VERSION)
+                    .contains(&protocol)
+                {
                     warn!(
                         "Received service protocol version is not supported: {}",
                         protocol
@@ -603,9 +723,35 @@ impl Service for AdminService {
 
                 Ok(())
             }
+            AdminMessage_Type::ABANDONED_CIRCUIT => {
+                let abandoned_circuit = admin_message.get_abandoned_circuit();
+                let circuit_id = abandoned_circuit.get_circuit_id();
+                let member_node_id = abandoned_circuit.get_member_node_id();
+
+                warn!(
+                    "Member {} has abandoned circuit {}",
+                    member_node_id, circuit_id
+                );
+                Ok(())
+            }
+            #[cfg(feature = "proposal-removal")]
+            AdminMessage_Type::REMOVED_PROPOSAL => {
+                let removed_proposal = admin_message.get_removed_proposal();
+                let circuit_id = removed_proposal.get_circuit_id();
+
+                warn!(
+                    "A prospective member has removed the proposal for circuit {}",
+                    circuit_id
+                );
+                Ok(())
+            }
             AdminMessage_Type::UNSET => Err(ServiceError::InvalidMessageFormat(Box::new(
                 AdminError::MessageTypeUnset,
             ))),
+            #[cfg(not(feature = "proposal-removal"))]
+            _ => Err(ServiceError::InvalidMessageFormat(Box::new(
+                AdminError::MessageTypeUnhandled,
+            ))),
         }
     }
 
@@ -669,13 +815,13 @@ impl AdminCommands for AdminServiceCommands {
 
     fn get_events_since(
         &self,
-        since_timestamp: &SystemTime,
+        since_event_id: &i64,
         event_type: &str,
     ) -> Result<Events, AdminServiceError> {
         self.shared
             .lock()
             .map_err(|_| AdminServiceError::general_error("Admin shared lock was lock poisoned"))?
-            .get_events_since(since_timestamp, event_type)
+            .get_events_since(since_event_id, event_type)
             .map_err(|err| {
                 AdminServiceError::general_error_with_source("Unable to get events", Box::new(err))
             })
@@ -716,53 +862,59 @@ fn supported_protocol_version(min: u32, max: u32) -> u32 {
         return 0;
     }
 
-    if min > ADMIN_PROTOCOL_VERSION {
+    if min > ADMIN_SERVICE_PROTOCOL_VERSION {
         info!(
             "Request requires newer version than can be provided: {}",
             min
         );
         return 0;
-    } else if max < ADMIN_PROTOCOL_VERSION {
+    } else if max < ADMIN_SERVICE_PROTOCOL_MIN {
         info!(
-            "Request requires older version than van be provided: {}",
+            "Request requires older version than can be provided: {}",
             max
         );
         return 0;
     }
 
-    ADMIN_PROTOCOL_VERSION
+    if max >= ADMIN_SERVICE_PROTOCOL_VERSION {
+        ADMIN_SERVICE_PROTOCOL_VERSION
+    } else if max > ADMIN_SERVICE_PROTOCOL_MIN {
+        max
+    } else if min > ADMIN_SERVICE_PROTOCOL_MIN {
+        min
+    } else {
+        ADMIN_SERVICE_PROTOCOL_MIN
+    }
 }
 
-#[cfg(test)]
+#[cfg(all(test, feature = "sqlite"))]
 mod tests {
     use super::*;
 
     use std::sync::mpsc::{channel, Sender};
     use std::time::{Duration, Instant};
 
-    use crate::circuit::{directory::CircuitDirectory, SplinterState};
+    use cylinder::{secp256k1::Secp256k1Context, Context};
+    use diesel::{
+        r2d2::{ConnectionManager as DieselConnectionManager, Pool},
+        sqlite::SqliteConnection,
+    };
+
+    use crate::admin::store::diesel::DieselAdminServiceStore;
+    use crate::circuit::routing::memory::RoutingTable;
     use crate::keys::insecure::AllowAllKeyPermissionManager;
     use crate::mesh::Mesh;
+    use crate::migrations::run_sqlite_migrations;
     use crate::network::auth::AuthorizationManager;
     use crate::network::connection_manager::authorizers::{Authorizers, InprocAuthorizer};
     use crate::network::connection_manager::ConnectionManager;
+    use crate::orchestrator::ServiceOrchestratorBuilder;
     use crate::peer::PeerManager;
     use crate::protos::admin;
     use crate::service::{error, ServiceNetworkRegistry, ServiceNetworkSender};
-    use crate::signing::{
-        hash::{HashSigner, HashVerifier},
-        Signer,
-    };
-    use crate::storage::get_storage;
+    use crate::threading::lifecycle::ShutdownHandle;
     use crate::transport::{inproc::InprocTransport, Transport};
 
-    const PUB_KEY: &[u8] = &[
-        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
-        25, 26, 27, 28, 29, 30, 31, 32,
-    ];
-
-    const STATE_DIR: &str = "/var/lib/splinter/";
-
     /// Test that a circuit creation creates the correct connections and sends the appropriate
     /// messages.
     #[test]
@@ -794,8 +946,8 @@ mod tests {
         authorizers.add_authorizer("inproc", inproc_authorizer);
         authorizers.add_authorizer("", authorization_manager.authorization_connector());
 
-        let mesh = Mesh::new(2, 2);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(2, 2);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(authorizers))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -804,7 +956,7 @@ mod tests {
             .expect("Unable to start Connection Manager");
         let connector = cm.connector();
 
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("test-node".to_string())
@@ -813,31 +965,51 @@ mod tests {
             .expect("Cannot start peer_manager");
         let peer_connector = peer_manager.connector();
 
-        let mut storage = get_storage("memory", CircuitDirectory::new).unwrap();
+        let connection_manager = DieselConnectionManager::<SqliteConnection>::new(":memory:");
+        let pool = Pool::builder()
+            .max_size(1)
+            .build(connection_manager)
+            .expect("Failed to build connection pool");
+
+        run_sqlite_migrations(&*pool.get().expect("Failed to get connection for migrations"))
+            .expect("Failed to run migrations");
 
-        let circuit_directory = storage.write().clone();
-        let state = SplinterState::new("memory".to_string(), circuit_directory);
         let orchestrator_connection = orchestrator_transport
             .connect("inproc://orchestator")
             .expect("failed to create connection");
-        let (orchestrator, _) = ServiceOrchestrator::new(vec![], orchestrator_connection, 1, 1, 1)
-            .expect("failed to create orchestrator");
+        let orchestrator = ServiceOrchestratorBuilder::new()
+            .with_connection(orchestrator_connection)
+            .build()
+            .expect("failed to create orchestrator")
+            .run()
+            .expect("failed to start orchestrator");
 
-        let (mut admin_service, _) = AdminService::new(
-            "test-node".into(),
-            orchestrator,
-            #[cfg(feature = "service-arg-validation")]
-            HashMap::new(),
-            peer_connector,
-            state,
-            Box::new(HashVerifier),
-            Box::new(MockAdminKeyVerifier),
-            Box::new(AllowAllKeyPermissionManager),
-            "memory",
-            STATE_DIR,
-            None,
-        )
-        .expect("Service should have been created correctly");
+        let context = Secp256k1Context::new();
+        let private_key = context.new_random_private_key();
+        let signer = context.new_signer(private_key);
+        let signature_verifier = context.new_verifier();
+
+        let table = RoutingTable::default();
+        let writer: Box<dyn RoutingTableWriter> = Box::new(table.clone());
+        let store = Box::new(DieselAdminServiceStore::new(pool));
+        let event_store = store.clone_boxed();
+
+        let mut admin_service_builder = AdminServiceBuilder::new();
+
+        admin_service_builder = admin_service_builder
+            .with_node_id("test-node".into())
+            .with_service_orchestrator(orchestrator)
+            .with_peer_manager_connector(peer_connector)
+            .with_admin_service_store(store)
+            .with_signature_verifier(signature_verifier)
+            .with_admin_key_verifier(Box::new(MockAdminKeyVerifier))
+            .with_key_permission_manager(Box::new(AllowAllKeyPermissionManager))
+            .with_routing_table_writer(writer)
+            .with_admin_event_store(event_store);
+
+        let mut admin_service = admin_service_builder
+            .build()
+            .expect("Service should have been created correctly");
 
         let (tx, rx) = channel();
         admin_service
@@ -853,6 +1025,7 @@ mod tests {
         proposed_circuit.set_durability(admin::Circuit_DurabilityType::NO_DURABILITY);
         proposed_circuit.set_circuit_management_type("test app auth handler".into());
         proposed_circuit.set_comments("test circuit".into());
+        proposed_circuit.set_display_name("test_display".into());
 
         proposed_circuit.set_members(protobuf::RepeatedField::from_vec(vec![
             splinter_node("test-node", &["inproc://someplace:8000".into()]),
@@ -868,12 +1041,17 @@ mod tests {
 
         let mut header = admin::CircuitManagementPayload_Header::new();
         header.set_action(admin::CircuitManagementPayload_Action::CIRCUIT_CREATE_REQUEST);
-        header.set_requester(PUB_KEY.into());
+        header.set_requester(
+            signer
+                .public_key()
+                .expect("Failed to get signer's public key")
+                .into_bytes(),
+        );
         header.set_requester_node_id("test-node".to_string());
 
         let mut payload = admin::CircuitManagementPayload::new();
 
-        payload.set_signature(HashSigner.sign(&payload.header).unwrap());
+        payload.set_signature(signer.sign(&payload.header).unwrap().take_bytes());
         payload.set_header(protobuf::Message::write_to_bytes(&header).unwrap());
         payload.set_circuit_create_request(request);
 
@@ -911,7 +1089,7 @@ mod tests {
             .admin_service_shared
             .lock()
             .unwrap()
-            .on_protocol_agreement("admin::other-node", 1)
+            .on_protocol_agreement("admin::other-node", ADMIN_SERVICE_PROTOCOL_VERSION)
             .expect("Unable to set protocol agreement");
 
         // wait up to 60 second for the proposed circuit message
@@ -955,11 +1133,15 @@ mod tests {
             envelope.take_circuit_create_request().take_circuit()
         );
 
-        peer_manager.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     fn splinter_node(node_id: &str, endpoints: &[String]) -> admin::SplinterNode {
