@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -62,17 +62,16 @@ use std::io;
 use std::sync::{Arc, RwLock};
 use std::time::Duration;
 
+use crate::collections::BiHashMap;
+use crate::error::InternalError;
 use crate::mesh::control::Control;
 pub use crate::mesh::control::{AddError, RemoveError};
 use crate::mesh::incoming::Incoming;
-pub use crate::mesh::matrix::{
-    MeshLifeCycle, MeshMatrixReceiver, MeshMatrixSender, MeshMatrixShutdown,
-};
+pub use crate::mesh::matrix::{MeshLifeCycle, MeshMatrixReceiver, MeshMatrixSender};
 use crate::mesh::outgoing::Outgoing;
-pub use crate::transport::matrix::ConnectionMatrixEnvelope as Envelope;
-
-use crate::collections::BiHashMap;
 use crate::mesh::reactor::Reactor;
+use crate::threading::lifecycle::ShutdownHandle;
+pub use crate::transport::matrix::ConnectionMatrixEnvelope as Envelope;
 use crate::transport::Connection;
 
 /// Wrapper around payload to include connection id
@@ -96,17 +95,6 @@ impl MeshState {
     }
 }
 
-#[derive(Clone)]
-pub struct MeshShutdownSignaler {
-    ctrl: Control,
-}
-
-impl MeshShutdownSignaler {
-    pub fn shutdown(&self) {
-        self.ctrl.shutdown();
-    }
-}
-
 /// A Connection reactor
 #[derive(Clone)]
 pub struct Mesh {
@@ -127,12 +115,6 @@ impl Mesh {
         }
     }
 
-    pub fn shutdown_signaler(&self) -> MeshShutdownSignaler {
-        MeshShutdownSignaler {
-            ctrl: self.ctrl.clone(),
-        }
-    }
-
     /// Add a new connection to the mesh, moving it to the background thread, and add the returned
     /// mesh id to the unique_ids map
     pub fn add(
@@ -175,7 +157,7 @@ impl Mesh {
         let id = envelope.id().to_string();
         if let Some(mesh_id) = state.unique_ids.get_by_key(&id) {
             match state.outgoings.get(mesh_id) {
-                Some(outgoing) => match outgoing.send(Vec::from(envelope)) {
+                Some(ref outgoing) => match outgoing.send(Vec::from(envelope)) {
                     Ok(()) => Ok(()),
                     Err(err) => Err(SendError::from_outgoing_send_error(err, id)),
                 },
@@ -252,10 +234,35 @@ impl Mesh {
         let mesh = self.clone();
         MeshMatrixReceiver::new(mesh)
     }
+}
 
-    /// Creates a MeshMatrixShutdown to shutdown this Mesh instance
-    pub fn get_matrix_shutdown(&self) -> MeshMatrixShutdown {
-        MeshMatrixShutdown::new(self.shutdown_signaler())
+impl ShutdownHandle for Mesh {
+    fn signal_shutdown(&mut self) {
+        self.ctrl.shutdown();
+    }
+
+    fn wait_for_shutdown(self) -> Result<(), InternalError> {
+        debug!("Waiting for Mesh to shutdown");
+        loop {
+            match self.recv() {
+                Ok(_) => debug!("Ignoring message"),
+                Err(err) => match err {
+                    RecvError::Shutdown => {
+                        debug!("Mesh has shutdown");
+                        return Ok(());
+                    }
+                    RecvError::PoisonedLock => {
+                        return Err(InternalError::with_message(
+                            "Mesh lock was poisoned while waiting for shutdown".to_string(),
+                        ))
+                    }
+                    RecvError::Disconnected => {
+                        debug!("Mesh has disconnected");
+                        return Ok(());
+                    }
+                },
+            }
+        }
     }
 }
 
@@ -566,7 +573,7 @@ mod tests {
         let mut listener = assert_ok(transport.listen("127.0.0.1:0"));
         let endpoint = listener.endpoint();
 
-        let mesh = Mesh::new(1, 1);
+        let mut mesh = Mesh::new(1, 1);
 
         let (client_ready_tx, client_ready_rx) = channel();
         let (server_ready_tx, server_ready_rx) = channel();
@@ -608,8 +615,7 @@ mod tests {
             InternalEnvelope::Shutdown => panic!("Should not have received shutdown"),
         }
 
-        let signaler = mesh.shutdown_signaler();
-        signaler.shutdown();
+        mesh.signal_shutdown();
 
         let envelope = assert_ok(incoming.recv());
         match envelope {
