@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -96,7 +96,7 @@ impl fmt::Display for AuthorizationActionError {
             AuthorizationActionError::InvalidMessageOrder(start, action) => {
                 write!(f, "Attempting to transition from {} via {}.", start, action)
             }
-            AuthorizationActionError::InternalError(msg) => f.write_str(msg),
+            AuthorizationActionError::InternalError(msg) => f.write_str(&msg),
         }
     }
 }
@@ -131,9 +131,9 @@ impl AuthorizationManager {
         let shared = Arc::new(Mutex::new(ManagedAuthorizations::new()));
 
         Ok(Self {
+            local_identity,
             thread_pool,
             shared,
-            local_identity,
         })
     }
 
