@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -29,6 +29,8 @@ pub use builder::ConnectionManagerBuilder;
 pub use error::{AuthorizerError, ConnectionManagerError};
 pub use notification::ConnectionManagerNotification;
 
+use crate::error::InternalError;
+use crate::threading::lifecycle::ShutdownHandle;
 use crate::threading::pacemaker;
 use crate::transport::matrix::{ConnectionMatrixLifeCycle, ConnectionMatrixSender};
 use crate::transport::{ConnectError, Connection, Transport};
@@ -178,29 +180,6 @@ impl ConnectionManager {
             sender: self.sender.clone(),
         }
     }
-
-    pub fn shutdown_signaler(&self) -> ShutdownSignaler {
-        ShutdownSignaler {
-            sender: self.sender.clone(),
-            pacemaker_shutdown_signaler: self.pacemaker.shutdown_signaler(),
-        }
-    }
-
-    /// Blocks until a connection manager has shutdown. This is meant to allow
-    /// a separate process to shutdown the connection manager via the shutdown
-    /// handle while another process waits for that process to complete.
-    pub fn await_shutdown(self) {
-        debug!("Shutting down connection manager pacemaker...");
-        self.pacemaker.await_shutdown();
-        debug!("Shutting down connection manager pacemaker (complete)");
-
-        if let Err(err) = self.join_handle.join() {
-            error!(
-                "Connection manager thread did not shutdown correctly: {:?}",
-                err
-            );
-        }
-    }
 }
 
 /// Connector is a client or handle to the connection manager and is used to
@@ -401,22 +380,27 @@ impl Connector {
     }
 }
 
-/// Sends a shutdown signal to the connection manager.
-#[derive(Clone)]
-pub struct ShutdownSignaler {
-    sender: Sender<CmMessage>,
-    pacemaker_shutdown_signaler: pacemaker::ShutdownSignaler,
-}
-
-impl ShutdownSignaler {
-    /// Signal the connection manager to shutdown.
-    pub fn shutdown(self) {
-        self.pacemaker_shutdown_signaler.shutdown();
+impl ShutdownHandle for ConnectionManager {
+    fn signal_shutdown(&mut self) {
+        self.pacemaker.shutdown_signaler().shutdown();
 
         if self.sender.send(CmMessage::Shutdown).is_err() {
             warn!("Connection manager is no longer running");
         }
     }
+
+    fn wait_for_shutdown(self) -> Result<(), InternalError> {
+        debug!("Shutting down connection manager pacemaker...");
+        self.pacemaker.await_shutdown();
+        debug!("Shutting down connection manager pacemaker (complete)");
+
+        self.join_handle.join().map_err(|err| {
+            InternalError::with_message(format!(
+                "Connection manager thread did not shutdown correctly: {:?}",
+                err
+            ))
+        })
+    }
 }
 
 /// Metadata describing a connection managed by the connection manager.
@@ -901,6 +885,7 @@ mod tests {
     use crate::network::auth::tests::negotiation_connection_auth;
     use crate::network::auth::AuthorizationManager;
     use crate::protos::network::{NetworkMessage, NetworkMessageType};
+    use crate::threading::lifecycle::ShutdownHandle;
     use crate::transport::inproc::InprocTransport;
     use crate::transport::socket::TcpTransport;
 
@@ -910,7 +895,7 @@ mod tests {
         transport.listen("inproc://test").unwrap();
         let mesh = Mesh::new(512, 128);
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -918,8 +903,9 @@ mod tests {
             .start()
             .expect("Unable to start Connection Manager");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     #[test]
@@ -932,7 +918,7 @@ mod tests {
         });
 
         let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -946,8 +932,9 @@ mod tests {
             .request_connection("inproc://test", "test_id")
             .expect("A connection could not be created");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     /// Test that adding the same connection twice is an idempotent operation
@@ -961,7 +948,7 @@ mod tests {
         });
 
         let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -979,8 +966,9 @@ mod tests {
             .request_connection("inproc://test", "test_id")
             .expect("A connection could not be re-requested");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     /// Test that heartbeats are correctly sent to inproc connections
@@ -996,7 +984,7 @@ mod tests {
             mesh_clone.add(conn, "test_id".to_string()).unwrap();
         });
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1019,8 +1007,9 @@ mod tests {
             NetworkMessageType::NETWORK_HEARTBEAT
         );
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     /// Test that heartbeats are correctly sent to tcp connections
@@ -1034,7 +1023,7 @@ mod tests {
 
         let (tx, rx) = mpsc::channel();
         thread::spawn(move || {
-            let mesh = Mesh::new(512, 128);
+            let mut mesh = Mesh::new(512, 128);
             let conn = listener.accept().unwrap();
             mesh.add(conn, "test_id".to_string()).unwrap();
 
@@ -1051,12 +1040,13 @@ mod tests {
 
             tx.send(()).expect("Could not send completion signal");
 
-            mesh.shutdown_signaler().shutdown();
+            mesh.signal_shutdown();
+            mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
         });
 
         let auth_mgr = AuthorizationManager::new("test_identity".into())
             .expect("Unable to create authorization pool");
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(auth_mgr.authorization_connector()))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1089,8 +1079,9 @@ mod tests {
         // wait for completion
         rx.recv().expect("Did not receive completion signal");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
         auth_mgr.shutdown_and_await();
     }
 
@@ -1103,7 +1094,7 @@ mod tests {
 
         let (tx, rx) = mpsc::channel();
         thread::spawn(move || {
-            let mesh = Mesh::new(512, 128);
+            let mut mesh = Mesh::new(512, 128);
             let conn = listener.accept().unwrap();
             mesh.add(conn, "test_id".to_string()).unwrap();
             negotiation_connection_auth(&mesh, "test_id", "some-peer");
@@ -1111,12 +1102,13 @@ mod tests {
             // wait for completion
             rx.recv().expect("Did not receive completion signal");
 
-            mesh.shutdown_signaler().shutdown();
+            mesh.signal_shutdown();
+            mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
         });
 
         let auth_mgr = AuthorizationManager::new("test_identity".into())
             .expect("Unable to create authorization pool");
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(auth_mgr.authorization_connector()))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1166,8 +1158,9 @@ mod tests {
 
         tx.send(()).expect("Could not send completion signal");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
         auth_mgr.shutdown_and_await();
     }
 
@@ -1176,7 +1169,7 @@ mod tests {
         let transport = Box::new(TcpTransport::default());
         let mesh = Mesh::new(512, 128);
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1191,8 +1184,9 @@ mod tests {
             .expect("Unable to remove connection");
 
         assert_eq!(None, endpoint_removed);
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     /// test_reconnect_raw_tcp
@@ -1212,7 +1206,7 @@ mod tests {
         let (tx, rx) = mpsc::channel();
         thread::spawn(move || {
             // accept incoming connection and add it to mesh2
-            let mesh2 = Mesh::new(512, 128);
+            let mut mesh2 = Mesh::new(512, 128);
             let conn = listener.accept().expect("Cannot accept connection");
             mesh2
                 .add(conn, "test_id".to_string())
@@ -1247,12 +1241,13 @@ mod tests {
             // wait for completion
             rx.recv().expect("Did not receive completion signal");
 
-            mesh2.shutdown_signaler().shutdown();
+            mesh2.signal_shutdown();
+            mesh2.wait_for_shutdown().expect("Unable to shutdown mesh");
         });
 
         let auth_mgr = AuthorizationManager::new("test_identity".into())
             .expect("Unable to create authorization pool");
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(auth_mgr.authorization_connector()))
             .with_matrix_life_cycle(mesh1.get_life_cycle())
             .with_matrix_sender(mesh1.get_sender())
@@ -1315,8 +1310,9 @@ mod tests {
 
         tx.send(()).expect("Could not send completion signal");
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
         auth_mgr.shutdown_and_await();
     }
 
@@ -1345,7 +1341,7 @@ mod tests {
             // block until done
             conn_rx.recv().unwrap();
         });
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_identity")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1388,8 +1384,9 @@ mod tests {
         conn_tx.send(()).unwrap();
         jh.join().unwrap();
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
     }
 
     /// Test that an inbound tcp connection can be add and removed from the network.o
@@ -1410,7 +1407,7 @@ mod tests {
         let (conn_tx, conn_rx) = mpsc::channel();
         let server_endpoint = endpoint.clone();
         let jh = thread::spawn(move || {
-            let mesh = Mesh::new(512, 128);
+            let mut mesh = Mesh::new(512, 128);
             let mut transport = Box::new(TcpTransport::default());
             let connection = transport.connect(&server_endpoint).unwrap();
 
@@ -1421,10 +1418,11 @@ mod tests {
 
             // block until done
             conn_rx.recv().unwrap();
-            mesh.shutdown_signaler().shutdown();
+            mesh.signal_shutdown();
+            mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
         });
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(auth_mgr.authorization_connector()))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1467,8 +1465,9 @@ mod tests {
         conn_tx.send(()).unwrap();
         jh.join().unwrap();
 
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
         auth_mgr.shutdown_and_await();
     }
 
