@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -40,8 +40,10 @@ use std::time::Instant;
 use uuid::Uuid;
 
 use crate::collections::{BiHashMap, RefMap};
+use crate::error::InternalError;
 use crate::network::connection_manager::ConnectionManagerNotification;
 use crate::network::connection_manager::{ConnectionManagerError, Connector};
+use crate::threading::lifecycle::ShutdownHandle;
 use crate::threading::pacemaker;
 
 pub use self::builder::PeerManagerBuilder;
@@ -129,7 +131,7 @@ pub(crate) enum PeerManagerRequest {
 pub struct PeerManager {
     join_handle: thread::JoinHandle<()>,
     sender: Sender<PeerManagerMessage>,
-    shutdown_signaler: ShutdownSignaler,
+    pacemaker_shutdown_signaler: pacemaker::ShutdownSignaler,
 }
 
 impl PeerManager {
@@ -146,6 +148,7 @@ impl PeerManager {
     /// * `identity` - The unique ID of the node this `PeerManager` belongs to
     /// * `strict_ref_counts` - Determines whether or not to panic when attempting to remove a
     ///   reference to peer that is not referenced.
+    #[deprecated(since = "0.5.1", note = "Please use PeerManagerBuilder instead")]
     pub fn new(
         connector: Connector,
         max_retry_attempts: Option<u64>,
@@ -183,6 +186,11 @@ impl PeerManager {
     /// handles notifications from the `ConnectionManager`.
     ///
     /// Returns a `PeerManagerConnector` that can be used to send requests to the `PeerManager`.
+    #[deprecated(
+        since = "0.5.1",
+        note = "Please use connector() instead. The PeerManagerBuilder starts up the PeerManager \
+         now"
+    )]
     pub fn start(&mut self) -> Result<PeerManagerConnector, PeerManagerError> {
         Ok(PeerManagerConnector::new(self.sender.clone()))
     }
@@ -191,31 +199,6 @@ impl PeerManager {
         PeerManagerConnector::new(self.sender.clone())
     }
 
-    /// Returns a `ShutdownHandle` for this `PeerManager`
-    pub fn shutdown_handle(&self) -> Option<ShutdownHandle> {
-        Some(ShutdownHandle::from(self.shutdown_signaler.clone()))
-    }
-
-    /// Returns a `ShutdownSignaler` for this `PeerManager`
-    pub fn shutdown_signaler(&self) -> ShutdownSignaler {
-        self.shutdown_signaler.clone()
-    }
-
-    /// Waits for the `PeerManager` thread to shutdown
-    pub fn await_shutdown(self) {
-        debug!("Shutting down peer manager...");
-        if let Err(err) = self.join_handle.join() {
-            error!("Peer manager thread did not shutdown correctly: {:?}", err);
-        }
-        debug!("Shutting down peer manager (complete)");
-    }
-
-    /// Sends a shutdown signal and waits for the `PeerManager` thread to shutdown
-    pub fn shutdown_and_wait(self) {
-        self.shutdown_signaler.shutdown();
-        self.await_shutdown();
-    }
-
     /// Private constructor used by the builder to start the peer manager
     #[allow(clippy::too_many_arguments)]
     // Allow clippy errors for too_many_arguments. This method is private and is in support of the
@@ -343,60 +326,32 @@ impl PeerManager {
                 ))
             })?;
 
-        let shutdown_signaler = ShutdownSignaler {
-            sender: sender.clone(),
-            pacemaker_shutdown_signaler,
-        };
-
         Ok(PeerManager {
-            shutdown_signaler,
             join_handle,
             sender,
+            pacemaker_shutdown_signaler,
         })
     }
 }
 
-/// Handles sending a shutdown message to the `PeerManager` and the `Pacemaker`.
-/// Deprecated, use ShutdownSignaler instead.
-#[derive(Clone)]
-pub struct ShutdownHandle {
-    sender: Sender<PeerManagerMessage>,
-    pacemaker_shutdown_signaler: pacemaker::ShutdownSignaler,
-}
-
-impl ShutdownHandle {
-    /// Sends a shutdown message to the `PeerManager` and `Pacemaker`
-    pub fn shutdown(&self) {
+impl ShutdownHandle for PeerManager {
+    fn signal_shutdown(&mut self) {
         self.pacemaker_shutdown_signaler.shutdown();
         if self.sender.send(PeerManagerMessage::Shutdown).is_err() {
             warn!("PeerManager is no longer running");
         }
     }
-}
 
-impl From<ShutdownSignaler> for ShutdownHandle {
-    fn from(signaler: ShutdownSignaler) -> Self {
-        ShutdownHandle {
-            sender: signaler.sender,
-            pacemaker_shutdown_signaler: signaler.pacemaker_shutdown_signaler,
-        }
-    }
-}
-
-/// Handles sending a shutdown message to the `PeerManager` and the `Pacemaker`
-#[derive(Clone)]
-pub struct ShutdownSignaler {
-    sender: Sender<PeerManagerMessage>,
-    pacemaker_shutdown_signaler: pacemaker::ShutdownSignaler,
-}
-
-impl ShutdownSignaler {
-    /// Sends a shutdown message to the `PeerManager` and `Pacemaker`
-    pub fn shutdown(&self) {
-        self.pacemaker_shutdown_signaler.shutdown();
-        if self.sender.send(PeerManagerMessage::Shutdown).is_err() {
-            warn!("PeerManager is no longer running");
-        }
+    fn wait_for_shutdown(self) -> Result<(), InternalError> {
+        debug!("Shutting down peer manager...");
+        self.join_handle.join().map_err(|err| {
+            InternalError::with_message(format!(
+                "Peer manager thread did not shutdown correctly: {:?}",
+                err
+            ))
+        })?;
+        debug!("Shutting down peer manager (complete)");
+        Ok(())
     }
 }
 
@@ -437,14 +392,14 @@ fn handle_request(
         }
         PeerManagerRequest::AddUnidentified { endpoint, sender } => {
             if sender
-                .send(add_unidentified(
+                .send(Ok(add_unidentified(
                     endpoint,
                     connector,
                     unreferenced_peers,
                     peer_remover,
                     peers,
                     ref_map,
-                ))
+                )))
                 .is_err()
             {
                 warn!("Connector dropped before receiving result of adding unidentified peer");
@@ -605,7 +560,7 @@ fn add_peer(
                     // if peer was added by endpoint, its peer metadata should be updated to
                     // include the full list of endpoints in this request
                     if unreferenced_peers.requested_endpoints.contains(endpoint)
-                        && endpoints.contains(endpoint)
+                        && endpoints.contains(&endpoint)
                     {
                         info!(
                             "Updating peer {} to include endpoints {:?}",
@@ -699,14 +654,14 @@ fn add_peer(
     };
 
     for endpoint in endpoints.iter() {
-        match connector.request_connection(endpoint, &connection_id) {
+        match connector.request_connection(&endpoint, &connection_id) {
             Ok(()) => {
                 active_endpoint = endpoint.to_string();
                 break;
             }
             // If the request_connection errored we will retry in the future
             Err(err) => {
-                log_connect_request_err(err, &peer_id, endpoint);
+                log_connect_request_err(err, &peer_id, &endpoint);
             }
         }
     }
@@ -730,11 +685,12 @@ fn add_unidentified(
     peer_remover: &PeerRemover,
     peers: &PeerMap,
     ref_map: &mut RefMap,
-) -> Result<EndpointPeerRef, PeerUnknownAddError> {
+) -> EndpointPeerRef {
     info!("Attempting to peer with peer by endpoint {}", endpoint);
     if let Some(peer_metadata) = peers.get_peer_from_endpoint(&endpoint) {
         // if there is peer in the peer_map, there is reference in the ref map
         ref_map.add_ref(peer_metadata.id.to_string());
+        EndpointPeerRef::new(endpoint, peer_remover.clone())
     } else {
         let connection_id = format!("{}", Uuid::new_v4());
         match connector.request_connection(&endpoint, &connection_id) {
@@ -746,8 +702,8 @@ fn add_unidentified(
         unreferenced_peers
             .requested_endpoints
             .push(endpoint.to_string());
+        EndpointPeerRef::new(endpoint, peer_remover.clone())
     }
-    Ok(EndpointPeerRef::new(endpoint, peer_remover.clone()))
 }
 
 fn remove_peer(
@@ -773,7 +729,7 @@ fn remove_peer(
                     peer_id
                 );
             } else {
-                return Err(PeerRefRemoveError::Remove(format!(
+                return Err(PeerRefRemoveError::RemoveError(format!(
                     "Failed to remove ref for peer {} from ref map: {}",
                     peer_id, err
                 )));
@@ -783,7 +739,7 @@ fn remove_peer(
 
     if let Some(removed_peer) = removed_peer {
         let peer_metadata = peers.remove(&removed_peer).ok_or_else(|| {
-            PeerRefRemoveError::Remove(format!(
+            PeerRefRemoveError::RemoveError(format!(
                 "Peer {} has already been removed from the peer map",
                 peer_id
             ))
@@ -801,11 +757,11 @@ fn remove_peer(
                 );
                 Ok(())
             }
-            Ok(None) => Err(PeerRefRemoveError::Remove(format!(
+            Ok(None) => Err(PeerRefRemoveError::RemoveError(format!(
                 "The connection for peer {}'s active endpoint ({}) has already been removed",
                 peer_id, peer_metadata.active_endpoint
             ))),
-            Err(err) => Err(PeerRefRemoveError::Remove(format!("{}", err))),
+            Err(err) => Err(PeerRefRemoveError::RemoveError(format!("{}", err))),
         }
     } else {
         // if the peer has not been fully removed, return OK
@@ -823,7 +779,7 @@ fn remove_peer_by_endpoint(
     let peer_metadata = match peers.get_peer_from_endpoint(&endpoint) {
         Some(peer_metadata) => peer_metadata,
         None => {
-            return Err(PeerRefRemoveError::Remove(format!(
+            return Err(PeerRefRemoveError::RemoveError(format!(
                 "Peer with endpoint {} has already been removed from the peer map",
                 endpoint
             )))
@@ -844,7 +800,7 @@ fn remove_peer_by_endpoint(
                     peer_metadata.id
                 );
             } else {
-                return Err(PeerRefRemoveError::Remove(format!(
+                return Err(PeerRefRemoveError::RemoveError(format!(
                     "Failed to remove ref for peer {} from ref map: {}",
                     peer_metadata.id, err
                 )));
@@ -853,7 +809,7 @@ fn remove_peer_by_endpoint(
     };
     if let Some(removed_peer) = removed_peer {
         let peer_metadata = peers.remove(&removed_peer).ok_or_else(|| {
-            PeerRefRemoveError::Remove(format!(
+            PeerRefRemoveError::RemoveError(format!(
                 "Peer with endpoint {} has already been removed from the peer map",
                 endpoint
             ))
@@ -872,11 +828,11 @@ fn remove_peer_by_endpoint(
                 );
                 Ok(())
             }
-            Ok(None) => Err(PeerRefRemoveError::Remove(format!(
+            Ok(None) => Err(PeerRefRemoveError::RemoveError(format!(
                 "The connection for peer {}'s active endpoint ({}) has already been removed",
                 peer_metadata.id, peer_metadata.active_endpoint
             ))),
-            Err(err) => Err(PeerRefRemoveError::Remove(format!("{}", err))),
+            Err(err) => Err(PeerRefRemoveError::RemoveError(format!("{}", err))),
         }
     } else {
         // if the peer has not been fully removed, return OK
@@ -935,10 +891,11 @@ fn handle_notifications(
                         if endpoint == &peer_metadata.active_endpoint {
                             continue;
                         }
-                        match connector.request_connection(endpoint, &peer_metadata.connection_id) {
+                        match connector.request_connection(&endpoint, &peer_metadata.connection_id)
+                        {
                             Ok(()) => break,
                             Err(err) => {
-                                log_connect_request_err(err, &peer_metadata.id, endpoint);
+                                log_connect_request_err(err, &peer_metadata.id, &endpoint);
                             }
                         }
                     }
@@ -1037,10 +994,10 @@ fn handle_disconnection(
 
             info!("Attempting to find available endpoint for {}", identity);
             for endpoint in peer_metadata.endpoints.iter() {
-                match connector.request_connection(endpoint, &peer_metadata.connection_id) {
+                match connector.request_connection(&endpoint, &peer_metadata.connection_id) {
                     Ok(()) => break,
                     Err(err) => {
-                        log_connect_request_err(err, &peer_metadata.id, endpoint);
+                        log_connect_request_err(err, &peer_metadata.id, &endpoint);
                     }
                 }
             }
@@ -1361,11 +1318,11 @@ fn retry_pending(
     for mut peer_metadata in to_retry {
         debug!("Attempting to peer with pending peer {}", peer_metadata.id);
         for endpoint in peer_metadata.endpoints.iter() {
-            match connector.request_connection(endpoint, &peer_metadata.connection_id) {
+            match connector.request_connection(&endpoint, &peer_metadata.connection_id) {
                 Ok(()) => peer_metadata.active_endpoint = endpoint.to_string(),
                 // If request_connection errored we will retry in the future
                 Err(err) => {
-                    log_connect_request_err(err, &peer_metadata.id, endpoint);
+                    log_connect_request_err(err, &peer_metadata.id, &endpoint);
                 }
             }
         }
@@ -1389,7 +1346,7 @@ fn retry_pending(
             }
             info!("Attempting to peer with peer by {}", endpoint);
             let connection_id = format!("{}", Uuid::new_v4());
-            match connector.request_connection(endpoint, &connection_id) {
+            match connector.request_connection(&endpoint, &connection_id) {
                 Ok(()) => (),
                 // If request_connection errored we will retry in the future
                 Err(err) => match err {
@@ -1477,6 +1434,7 @@ pub mod tests {
         AuthorizationResult, Authorizer, AuthorizerError, ConnectionManager,
     };
     use crate::protos::network::{NetworkMessage, NetworkMessageType};
+    use crate::threading::lifecycle::ShutdownHandle;
     use crate::transport::inproc::InprocTransport;
     use crate::transport::raw::RawTransport;
     use crate::transport::{Connection, Transport};
@@ -1495,8 +1453,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1505,7 +1463,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1538,11 +1496,15 @@ pub mod tests {
                 }
         );
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that a call to add_peer_ref, where the authorizer returns an different id than
@@ -1562,8 +1524,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("different_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1572,7 +1534,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector.clone())
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1613,11 +1575,15 @@ pub mod tests {
             .expect("Unable to list connections")
             .is_empty());
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that a call to add_peer_ref with a peer with multiple endpoints is successful, even if
@@ -1636,8 +1602,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1646,7 +1612,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1685,11 +1651,15 @@ pub mod tests {
                 }
         );
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that the same peer can be added multiple times.
@@ -1706,8 +1676,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1716,7 +1686,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1755,11 +1725,15 @@ pub mod tests {
 
         assert_eq!(peer_ref.peer_id(), "test_peer");
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that list_peer returns the correct list of peers
@@ -1784,8 +1758,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new_multiple(&[
                 "test_peer",
                 "next_peer",
@@ -1797,7 +1771,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1857,11 +1831,15 @@ pub mod tests {
             vec!["next_peer".to_string(), "test_peer".to_string()]
         );
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that list_peer returns the correct list of connection IDs
@@ -1884,8 +1862,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new_multiple(&[
                 "test_peer",
                 "next_peer",
@@ -1897,7 +1875,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -1954,11 +1932,15 @@ pub mod tests {
 
         assert!(peers.get_by_key("test_peer").is_some());
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that when a PeerRef is dropped, a remove peer request is properly sent and the peer
@@ -1979,8 +1961,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -1989,7 +1971,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -2038,11 +2020,15 @@ pub mod tests {
 
         assert_eq!(peer_list, Vec::<String>::new());
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that when a EndpointPeerRef is dropped, a remove peer request is properly sent and the
@@ -2067,8 +2053,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -2078,7 +2064,7 @@ pub mod tests {
 
         // let (finish_tx, fininsh_rx) = channel();
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -2139,11 +2125,15 @@ pub mod tests {
 
         assert_eq!(peer_list, Vec::<String>::new());
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that if a peer's endpoint disconnects and does not reconnect during a set timeout, the
@@ -2164,8 +2154,8 @@ pub mod tests {
             .listen("tcp://localhost:0")
             .expect("Cannot listen for connections");
         let endpoint = listener.endpoint();
-        let mesh1 = Mesh::new(512, 128);
-        let mesh2 = Mesh::new(512, 128);
+        let mut mesh1 = Mesh::new(512, 128);
+        let mut mesh2 = Mesh::new(512, 128);
 
         let mut listener2 = transport
             .listen("tcp://localhost:0")
@@ -2204,10 +2194,11 @@ pub mod tests {
 
             rx.recv().unwrap();
 
-            mesh2.shutdown_signaler().shutdown();
+            mesh2.signal_shutdown();
+            mesh2.wait_for_shutdown().expect("Unable to shutdown mesh");
         });
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new_multiple(&[
                 "test_peer",
                 "test_peer",
@@ -2220,7 +2211,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_max_retry_attempts(1)
@@ -2280,11 +2271,15 @@ pub mod tests {
         tx.send(()).unwrap();
 
         jh.join().unwrap();
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh1.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh1.signal_shutdown();
+        mesh1.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that the PeerManager can be started and stopped
@@ -2292,8 +2287,8 @@ pub mod tests {
     fn test_peer_manager_shutdown() {
         let transport = Box::new(InprocTransport::default());
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -2302,7 +2297,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -2310,11 +2305,15 @@ pub mod tests {
             .start()
             .expect("Cannot start peer_manager");
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that the PeerManager can receive incoming peer requests and handle them appropriately.
@@ -2327,8 +2326,8 @@ pub mod tests {
         let mut transport = InprocTransport::default();
         let mut listener = transport.listen("inproc://test").unwrap();
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -2351,7 +2350,7 @@ pub mod tests {
             subs_rx.recv().expect("unable to get notification");
         });
 
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -2384,11 +2383,15 @@ pub mod tests {
 
         assert_eq!(peer_list, vec!["test_peer".to_string()]);
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that the PeerManager can be started with the deprecated PeerManager::new() and
@@ -2398,8 +2401,8 @@ pub mod tests {
     fn test_peer_manager_no_builder() {
         let transport = Box::new(InprocTransport::default());
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -2409,16 +2412,22 @@ pub mod tests {
 
         let connector = cm.connector();
 
+        #[allow(deprecated)]
         let mut peer_manager =
             PeerManager::new(connector, Some(1), Some(1), "my_id".to_string(), true);
 
+        #[allow(deprecated)]
         peer_manager.start().expect("Cannot start peer_manager");
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     // Test that a subscriber can convert the PeerManagerNotification to another type
@@ -2435,8 +2444,8 @@ pub mod tests {
             listener.accept().unwrap();
         });
 
-        let mesh = Mesh::new(512, 128);
-        let cm = ConnectionManager::builder()
+        let mut mesh = Mesh::new(512, 128);
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test_peer")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -2445,7 +2454,7 @@ pub mod tests {
             .expect("Unable to start Connection Manager");
 
         let connector = cm.connector();
-        let peer_manager = PeerManager::builder()
+        let mut peer_manager = PeerManager::builder()
             .with_connector(connector)
             .with_retry_interval(1)
             .with_identity("my_id".to_string())
@@ -2475,11 +2484,15 @@ pub mod tests {
                 })
         );
 
-        peer_manager.shutdown_signaler().shutdown();
-        cm.shutdown_signaler().shutdown();
-        peer_manager.await_shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        peer_manager.signal_shutdown();
+        cm.signal_shutdown();
+        peer_manager
+            .wait_for_shutdown()
+            .expect("Unable to shutdown peer manager");
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
     }
 
     #[derive(PartialEq)]
