@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -217,7 +217,7 @@ where
                     message_receiver,
                     service_msg_dispatcher_sender,
                 ) {
-                    error!("Shutting down service interconnect recevier: {}", err);
+                    error!("Shutting down service interconnect receiver: {}", err);
                 }
             })
             .map_err(|err| {
@@ -300,7 +300,7 @@ where
         // If we have the service, pass message to dispatcher, else print error
         if let Some(service_id) = service_id {
             let mut component_msg: ComponentMessage =
-                match Message::parse_from_bytes(envelope.payload()) {
+                match Message::parse_from_bytes(&envelope.payload()) {
                     Ok(msg) => msg,
                     Err(err) => {
                         error!("Unable to dispatch message: {}", err);
@@ -438,6 +438,7 @@ pub mod tests {
     };
     use crate::protos::service;
     use crate::service::network::{ServiceConnectionManager, ServiceConnectionNotification};
+    use crate::threading::lifecycle::ShutdownHandle;
     use crate::transport::{inproc::InprocTransport, Connection, Transport};
 
     // Verify that the ServiceInterconnect properly receives messages from services, passes them to
@@ -484,10 +485,10 @@ pub mod tests {
         let mut listener = transport
             .listen("inproc://test")
             .expect("Cannot listen for connections");
-        let mesh1 = Mesh::new(512, 128);
-        let mesh2 = Mesh::new(512, 128);
+        let mut mesh1 = Mesh::new(512, 128);
+        let mut mesh2 = Mesh::new(512, 128);
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test-service")))
             .with_matrix_life_cycle(mesh1.get_life_cycle())
             .with_matrix_sender(mesh1.get_sender())
@@ -522,29 +523,38 @@ pub mod tests {
             let envelope = Envelope::new("test_id".to_string(), message_bytes);
             mesh2.send(envelope).expect("Unable to send message");
 
-            // Verify mesh received the same network echo back
-            let envelope = mesh2.recv().expect("Cannot receive message");
-            let mut network_msg: ComponentMessage = Message::parse_from_bytes(&envelope.payload())
-                .expect("Cannot parse ComponentMessage");
-
-            if network_msg.get_message_type() == ComponentMessageType::COMPONENT_HEARTBEAT {
-                // try to get the service message
+            let mut err = Some("Did not receive message".to_string());
+            for _attempt in 0..10 {
+                // Verify mesh received the same network echo back
                 let envelope = mesh2.recv().expect("Cannot receive message");
-                network_msg = Message::parse_from_bytes(&envelope.payload())
+                let network_msg: ComponentMessage = Message::parse_from_bytes(&envelope.payload())
                     .expect("Cannot parse ComponentMessage");
+
+                match network_msg.get_message_type() {
+                    // ignore heartbeats
+                    ComponentMessageType::COMPONENT_HEARTBEAT => (),
+                    ComponentMessageType::SERVICE => {
+                        let echo: service::ServiceProcessorMessage =
+                            Message::parse_from_bytes(network_msg.get_payload()).unwrap();
+
+                        assert_eq!(echo.get_payload().to_vec(), b"test_retrieve".to_vec());
+                        err = None;
+                        break;
+                    }
+                    _ => {
+                        err = Some(format!(
+                            "Unexpected message type {:?}",
+                            network_msg.get_message_type()
+                        ));
+                    }
+                }
             }
 
-            assert_eq!(
-                network_msg.get_message_type(),
-                ComponentMessageType::SERVICE
-            );
-
-            let echo: service::ServiceProcessorMessage =
-                Message::parse_from_bytes(network_msg.get_payload()).unwrap();
-
-            assert_eq!(echo.get_payload().to_vec(), b"test_retrieve".to_vec());
-
-            mesh2.shutdown_signaler().shutdown();
+            mesh2.signal_shutdown();
+            mesh2.wait_for_shutdown().expect("Unable to shutdown mesh");
+            if let Some(err) = err {
+                panic!("{}", err);
+            }
         });
         let (dispatcher_sender, dispatcher_receiver) = dispatch_channel();
         let interconnect = ServiceInterconnectBuilder::new()
@@ -562,15 +572,13 @@ pub mod tests {
         let handler = ComponentTestHandler::new(&[b"test_retrieve"]);
         dispatcher.set_handler(Box::new(handler));
 
-        let dispatch_loop = DispatchLoopBuilder::new()
+        let mut dispatch_loop = DispatchLoopBuilder::new()
             .with_dispatcher(dispatcher)
             .with_thread_name("ServiceDispatchLoop".to_string())
             .with_dispatch_channel((dispatcher_sender, dispatcher_receiver))
             .build()
             .expect("Unable to create service dispatch loop");
 
-        let dispatch_shutdown = dispatch_loop.shutdown_signaler();
-
         let conn = listener.accept().expect("Cannot accept connection");
         connector.add_inbound_connection(conn).unwrap();
 
@@ -579,10 +587,15 @@ pub mod tests {
         join_handle.join().unwrap();
 
         service_conn_mgr.shutdown_and_wait();
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
-        dispatch_shutdown.shutdown();
-        mesh1.shutdown_signaler().shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        dispatch_loop.signal_shutdown();
+        dispatch_loop
+            .wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh1.signal_shutdown();
+        mesh1.wait_for_shutdown().expect("Unable to shutdown mesh");
         interconnect.shutdown_and_wait();
     }
 
@@ -592,9 +605,9 @@ pub mod tests {
     #[test]
     fn test_service_interconnect_shutdown() {
         let transport = Box::new(InprocTransport::default());
-        let mesh = Mesh::new(512, 128);
+        let mut mesh = Mesh::new(512, 128);
 
-        let cm = ConnectionManager::builder()
+        let mut cm = ConnectionManager::builder()
             .with_authorizer(Box::new(NoopAuthorizer::new("test-service")))
             .with_matrix_life_cycle(mesh.get_life_cycle())
             .with_matrix_sender(mesh.get_sender())
@@ -619,9 +632,11 @@ pub mod tests {
             .expect("Unable to build ServiceInterconnect");
 
         service_conn_mgr.shutdown_and_wait();
-        cm.shutdown_signaler().shutdown();
-        cm.await_shutdown();
-        mesh.shutdown_signaler().shutdown();
+        cm.signal_shutdown();
+        cm.wait_for_shutdown()
+            .expect("Unable to shutdown connection manager");
+        mesh.signal_shutdown();
+        mesh.wait_for_shutdown().expect("Unable to shutdown mesh");
         interconnect.shutdown_and_wait();
     }
 
